/*
 * author: David Vaz (davidvaz@ncc.up.pt)
 *
 * date: 2006/10/03
 *
 * purpose: a pan object to overlay in the map
 *
 * IDEA:
 * unlike kaZoomer, kaPan does not create his own div
 * allowing to define the position in plain CSS
 *
 * There is also a function (align) that aligns both tools
 *
 */

/******************************************************************************
 * to use this tool:
 *
 * 1) add a script tag to your page
 * <script type="text/javascript" src="kaPan.js"></script>
 *
 * 2) create a container div
 * <div id="pan"></div>
 *
 * 3) create a new instance of kaPan
 * myKaPan = new kaPan(myKaMap,'pan');
 *
 * 4) optionally you can align kaPan with kaZoomer after
 *    KAMAP_MAP_INITIALIZED event
 * myKaPan.align(myKaZoom));
 *
 *****************************************************************************/

/******************************************************************************
 * kaPan
 *
 * class to handle the Pan overlay
 *
 * oKaMap - the ka-Map instance to draw the zoomer on
 *
 * szID - string with the div id for this tool
 *
 *****************************************************************************/

function kaPan (okaMap, szID)
{
  this.kaMap = okaMap;
  this.id = szID;
  this.panControlObj = this.kaMap.getRawObject( szID );

  this.opacity = 50;

  /*
   * configuration settings regarding overall style
   * all icons must have the same size
   */

  this.szPanUpImg = 'new_images/arrow_up.png';
  this.szPanDownImg = 'new_images/arrow_down.png';
  this.szPanLeftImg = 'new_images/arrow_left.png';
  this.szPanRightImg = 'new_images/arrow_right.png';
  this.szPanCenterImg = 'new_images/arrow_center.png';

  this.szPanUpAlt = "A";
  this.szPanDownAlt = "V";
  this.szPanLeftAlt = "<-";
  this.szPanRightAlt = "->";
  this.szPanCenterAlt = "+";

  this.panWidth = 16;
  this.panHeight = 16;
  this.panSpacing = 2;

  this.nStep = 16;

  /* prototypes */
  this.draw = kaPan_draw;
  this.align = kaPan_align;

  /*auxiliar prototypes*/
  this.draw_button = kaPan_draw_button;
  this.update = kaPan_update;

  this.kaMap.registerForEvent( KAMAP_MAP_INITIALIZED,
			       this, this.draw );
}

function kaPan_draw()
{
  this.panControlObj.onclick = null;
  this.panControlObj.style.width = 3 * this.panWidth
    + 2 * this.panSpacing + 'px';
  this.panControlObj.style.height = 3 * this.panHeight
    + 2 * this.panSpacing + 'px';
  this.panControlObj.style.opacity = this.opacity/100;
  this.panControlObj.style.mozOpacity = this.opacity/100;
  this.panControlObj.style.filter = "Alpha(opacity="+this.opacity+")";
  this.panControlObj.style.cursor = 'auto';
  this.panControlObj.onmouseover = kaPan_onmouseover;
  this.panControlObj.onmouseout = kaPan_onmouseout;
  this.panControlObj.kaPan = this;


  this.draw_button ('panUp', this.szPanUpImg,
		    this.szPanUpAlt, kaPan_eventPanUp,
		    0, this.panWidth + this.panSpacing);

  this.draw_button ('panRight', this.szPanRightImg,
		    this.szPanRightAlt, kaPan_eventPanRight,
		    this.panHeight + this.panSpacing,
		    2 * (this.panWidth + this.panSpacing));

  this.draw_button ('panDown', this.szPanDownImg,
		    this.szPanDownAlt, kaPan_eventPanDown,
		    2 * (this.panHeight + this.panSpacing),
		    this.panWidth + this.panSpacing);

  this.draw_button ('panLeft', this.szPanLeftImg,
		    this.szPanLeftAlt, kaPan_eventPanLeft,
		    this.panHeight + this.panSpacing, 0);

  this.draw_button ('panCenter', this.szPanCenterImg,
		    this.szPanCenterAlt,
		    kaPan_eventPanCenter,
		    this.panHeight + this.panSpacing,
		    this.panWidth + this.panSpacing);
}

function kaPan_align(zoomControl)
{
  zoomControl.moveBy(getObjectLeft(this.id)
		     + (getObjectWidth(this.id)
			- getObjectWidth(zoomControl.id)) / 2,
		     getObjectTop(this.id)
		     + getObjectHeight(this.id) + 2);
}

function kaPan_draw_button (name, szImg, szAlt,
			    event, top, left)
{
  var p = document.createElement ('div');
  p.id = name;
  p.style.position = 'absolute';
  p.style.top = top + 'px';
  p.style.left = left + 'px';
  p.style.width = this.panWidth + 'px';
  p.style.height = this.panHeight + 'px';
  p.innerHTML = '<img src="' + szImg
    + '" border="0" width="' + this.panWidth + '" height="'
    + this.panHeight + '" alt="' + szAlt + '">';
  p.kaPan = this;
  if (event != null)
    p.onclick = event;
  this.panControlObj.appendChild(p);
}


function kaPan_update (e)
{
  this.kaMap.triggerEvent (KAMAP_EXTENTS_CHANGED,
			   this.kaMap.getGeoExtents ());
}

function kaPan_eventPanUp (e)
{
  this.kaPan.kaMap.moveBy (0, this.kaPan.nStep);
  this.kaPan.update ();
}

function kaPan_eventPanRight (e)
{
  this.kaPan.kaMap.moveBy (-this.kaPan.nStep, 0);
  this.kaPan.update ();
}

function kaPan_eventPanDown (e)
{
  this.kaPan.kaMap.moveBy (0, -this.kaPan.nStep);
  this.kaPan.update ();
}

function kaPan_eventPanLeft (e)
{
  this.kaPan.kaMap.moveBy (this.kaPan.nStep, 0);
  this.kaPan.update ();
}

function kaPan_eventPanCenter (e)
{
  var ex = this.kaPan.kaMap.getCurrentMap().defaultExtents;
  this.kaPan.kaMap.zoomToExtents(ex[0],ex[1],ex[2],ex[3]);
}

function kaPan_onmouseover( e )
{
    this.style.opacity = 1;
    this.style.mozOpacity = 1;
    this.style.filter = "Alpha(opacity=100)";
}

function kaPan_onmouseout( e )
{
    this.style.opacity = this.kaPan.opacity/100;
    this.style.mozOpacity = this.kaPan.opacity/100;
    this.style.filter = "Alpha(opacity="+this.kaPan.opacity+")";
}
