/**********************************************************************
 *
 * $Id: kaZoomer.js,v 1.2 2006/02/07 03:19:55 pspencer Exp $
 *
 * purpose: a sliding zoom control intended to be put in the map
 *
 * author: Paul Spencer (pspencer@dmsolutions.ca)
 *
 * The original kaZoomer code was written by DM Solutions Group.
 *
 * TODO:
 *
 **********************************************************************
 *
 * Copyright (c) 2005, DM Solutions Group Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************/

/******************************************************************************
 * kaZoomer
 *
 * class to handle the zoom overlay
 *
 * oKaMap - the ka-Map instance to draw the zoomer on
 *
 *****************************************************************************/
function kaZoomer(oKaMap)
{
    this.kaMap = oKaMap;

    this.id = 'zoomControl';

    /* configuration settings regarding overall style */

    this.zoomControlWidth = 16;
    this.zoomControlHeight = 16;
    this.zoomControlTrackSpacing = 2;

    this.zoomThumbWidth = 16;
    this.zoomThumbHeight = 16;

    /* top middle and bottom track must have the same size */
    this.zoomTrackWidth = 16;
    this.zoomTrackHeight = 16;

    /* widget images */
    this.szThumbImg = 'new_images/slider_button.png';
    this.szTrackTopImg = 'new_images/slider_tray_top.png';
    this.szTrackBottomImg = 'new_images/slider_tray_bottom.png';
    this.szTrackFillImg = 'new_images/slider_tray_fill.png';
    this.szControlZoomIn = 'new_images/slider_button_zoomin.png';
    this.szControlZoomOut = 'new_images/slider_button_zoomout.png';

    this.zIndex = 300;
    this.opacity = 50;

    this.left = 0;
    this.top = 0;

    this.zoomControlObj = null;

    //prototypes
    this.draw = kaZoomer_draw;
    this.update = kaZoomer_update;
    this.moveBy = kaZoomer_moveBy;

    this.kaMap.registerForEvent( KAMAP_MAP_INITIALIZED, this, this.draw );
}

/* move the tool by (x,y) px */
function kaZoomer_moveBy(x,y)
{
    dd.elements.zoomControl.moveBy(x,y);
}

//handle a map zoom change from another tool
function kaZoomer_update()
{
    var nThumbHeight = dd.elements.zoomTrack.div.elementHeight;
    var nTrackTop = dd.elements.zoomTrack.y;
    var oKaMap = dd.elements.zoomTrack.div.kaZoomer.kaMap;
    var oMap = oKaMap.getCurrentMap();
    var nCurrentScale = parseInt(oMap.currentScale) + 1; //array is zero based
    var nScales = oMap.getScales().length;
    var nTrackHeight = this.zoomTrackHeight * nScales;
    var nPos = (nScales-nCurrentScale)*nThumbHeight;
    dd.elements.zoomThumb.moveTo(dd.elements.zoomThumb.x,nTrackTop + nPos);
}

//set up the slider UI.
function kaZoomer_draw()
{
    //get scale info for the current map.
    var oMap = this.kaMap.getCurrentMap();
    var nScales = oMap.getScales().length;
    var nCurrentScale = oMap.currentScale;
    var nTrackHeight = this.zoomTrackHeight * nScales;
    var nTrackMaxPosition = this.zoomTrackHeight * (nScales - 1);
    var nInitialPosition = dd.Int(this.zoomTrackHeight *
                                  (nScales - nCurrentScale - 1));

    //container div
    this.zoomControlObj = document.createElement('div');
    this.zoomControlObj.id = this.id;
    this.zoomControlObj.style.position = 'absolute';
    if (this.left != null)
      this.zoomControlObj.style.left = this.left + 'px';
    if (this.top != null)
      this.zoomControlObj.style.top = this.top + 'px';
    this.zoomControlObj.style.width = this.zoomControlWidth + "px";
    this.zoomControlObj.style.height = (nTrackHeight + 2
					* (this.zoomControlHeight
					   + this.zoomControlTrackSpacing))
                                       + "px";
    this.zoomControlObj.style.opacity = this.opacity/100;
    this.zoomControlObj.style.mozOpacity = this.opacity/100;
    this.zoomControlObj.style.filter = "Alpha(opacity="+this.opacity+")";
    this.zoomControlObj.style.cursor = 'auto';
    this.zoomControlObj.style.zIndex = this.zIndex;
    this.zoomControlObj.onmouseover = kaZoomer_onmouseover;
    this.zoomControlObj.onmouseout = kaZoomer_onmouseout;
    this.zoomControlObj.kaZoomer = this;
    this.kaMap.domObj.appendChild(this.zoomControlObj);

    //draw the widget
    var oZoomTrack = document.createElement( 'div' );
    oZoomTrack.id = 'zoomTrack';
    oZoomTrack.kaZoomer = this;
    oZoomTrack.style.position = 'absolute';
    oZoomTrack.style.left = '0px';
    oZoomTrack.style.top = (this.zoomControlHeight
			    + this.zoomControlTrackSpacing) + 'px';
    oZoomTrack.style.width = this.zoomTrackWidth + 'px';
    oZoomTrack.style.height = nTrackHeight + 'px';
    oZoomTrack.style.zIndex = this.zIndex + 1;
    oZoomTrack.elementHeight = this.zoomTrackHeight;
    oZoomTrack.onclick = kaZoomer_zoomTo;
    this.zoomControlObj.appendChild(oZoomTrack);

    if (nScales >= 2)
      {
	var oZoomTrackTop = document.createElement( 'div' );
	oZoomTrackTop.id = 'zoomTrackTop';
	oZoomTrackTop.style.position = 'absolute';
	oZoomTrackTop.style.left = '0px';
	oZoomTrackTop.style.top = (this.zoomControlHeight + this.zoomControlTrackSpacing) + 'px';
	oZoomTrackTop.style.width = this.zoomTrackWidth + 'px';
	oZoomTrackTop.style.height = this.zoomTrackHeight + 'px';
	oZoomTrackTop.innerHTML = '<img src="' + this.szTrackTopImg +'" border="0" width="'
	  + this.zoomTrackWidth + '" height="' + this.zoomTrackHeight + '">';
	this.zoomControlObj.appendChild(oZoomTrackTop);

	if (nScales >= 3)
	  {
	    var oZoomTrackMiddle = document.createElement( 'div' );
	    oZoomTrackMiddle.id = 'zoomTrackMiddle';
	    oZoomTrackMiddle.style.position = 'absolute';
	    oZoomTrackMiddle.style.left = '0px';
	    oZoomTrackMiddle.style.top = (this.zoomControlHeight + this.zoomControlTrackSpacing
					  + this.zoomTrackHeight) + 'px';
	    oZoomTrackMiddle.style.height = (parseInt(nTrackHeight) - 2 * this.zoomTrackHeight) + 'px';
	    oZoomTrackMiddle.style.width = this.zoomTrackWidth + 'px';
	    oZoomTrackMiddle.style.backgroundColor = "#acacac";
	    oZoomTrackMiddle.style.backgroundImage = 'url(' + this.szTrackFillImg + ')';
	    this.zoomControlObj.appendChild(oZoomTrackMiddle);
	  }

	var oZoomTrackBottom = document.createElement( 'div' );
	oZoomTrackBottom.id = 'zoomTrackBottom';
	oZoomTrackBottom.style.position = 'absolute';
	oZoomTrackBottom.style.left = '0px';
	oZoomTrackBottom.style.top = (this.zoomControlHeight + this.zoomControlTrackSpacing
				      + (nTrackHeight - this.zoomTrackHeight))  + 'px';
	oZoomTrackBottom.style.width = this.zoomTrackWidth + 'px';
	oZoomTrackBottom.style.height = this.zoomTrackHeight + 'px';
	oZoomTrackBottom.innerHTML = '<img src="' + this.szTrackBottomImg +'" border="0" width="'
	  + this.zoomTrackWidth + '" height="' + this.zoomTrackHeight + '">';
	this.zoomControlObj.appendChild(oZoomTrackBottom);

      }

    var oZoomThumb = document.createElement( 'div' );
    oZoomThumb.id = 'zoomThumb';
    oZoomThumb.style.position = 'absolute';
    oZoomThumb.style.height = this.zoomTrackHeight + 'px';
    oZoomThumb.style.width = this.zoomThumbWidth + 'px';
    oZoomThumb.style.backgroundColor = "#888888";
    oZoomThumb.innerHTML = '<img src="' + this.szThumbImg +'" border="0" width="'
      + this.zoomTrackWidth + '" height="' + this.zoomTrackHeight + '">';
    this.zoomControlObj.appendChild(oZoomThumb);

    //add +/- labels
    var oZoomIn = document.createElement('div');
    oZoomIn.id = 'zoomIn';
    oZoomIn.style.position = 'absolute';
    oZoomIn.style.top = '0px';
    oZoomIn.style.left = '0px';
    oZoomIn.style.width = this.zoomControlWidth + 'px';
    oZoomIn.style.height = this.zoomControlHeight + 'px';
    oZoomIn.kaZoomer = this;
    oZoomIn.onclick = kaZoomer_zoomIn;
    oZoomIn.innerHTML= '<img src="' + this.szControlZoomIn + '" border="0" width="'
      + this.zoomControlWidth + ' height="' + this.zoomControlHeight + '">';
    this.zoomControlObj.appendChild(oZoomIn);

    var oZoomOut = document.createElement('div');
    oZoomOut.id = 'zoomOut';
    oZoomOut.style.position = 'absolute';
    oZoomOut.style.top = (this.zoomControlHeight + 2 * this.zoomControlTrackSpacing
			  + nTrackHeight) + 'px';
    oZoomOut.style.left = '0px';
    oZoomOut.style.width = this.zoomControlWidth + 'px';
    oZoomOut.style.height = this.zoomControlHeight + 'px';
    oZoomOut.kaZoomer = this;
    oZoomOut.onclick = kaZoomer_zoomOut;
    oZoomOut.innerHTML= '<img src="' + this.szControlZoomOut + '" border="0" width="'
      + this.zoomControlWidth + ' height="' + this.zoomControlHeight + '">';
    this.zoomControlObj.appendChild(oZoomOut);

    //set up drag and drop
    ADD_DHTML('zoomThumb'+MAXOFFTOP+0+MAXOFFBOTTOM+nTrackMaxPosition+VERTICAL);
    ADD_DHTML('zoomTrack'+NO_DRAG);
    ADD_DHTML('zoomControl'+NO_DRAG); /*to allow repositioning*/

    dd.elements.zoomThumb.moveTo(dd.elements.zoomTrack.x, dd.elements.zoomTrack.y + nInitialPosition);
    dd.elements.zoomThumb.setZ(dd.elements.zoomTrack.z+1);

    dd.elements.zoomControl.addChild('zoomTrack');
    dd.elements.zoomTrack.addChild('zoomThumb');

    dd.elements.zoomThumb.defx = dd.elements.zoomTrack.x;
    dd.elements.zoomThumb.defy = dd.elements.zoomTrack.y;

    dd.elements.zoomThumb.my_DropFunc = kaZoomer_DropFunc;

    this.kaMap.registerForEvent (KAMAP_SCALE_CHANGED, this, this.update);
}

//wz_dragdrop.js overriden function for responding to a release of the slider
function kaZoomer_DropFunc()
{
    //move thumb to closest scale marker
    var nTrackTop = dd.elements.zoomTrack.y;

    var nThumbTop = dd.elements.zoomThumb.y - nTrackTop;
    var nThumbHeight = dd.elements.zoomTrack.div.elementHeight;

    var nNearestIndex = Math.round(nThumbTop / nThumbHeight);
    dd.elements.zoomThumb.moveTo(dd.elements.zoomThumb.x,nTrackTop +(nNearestIndex*nThumbHeight));

    //perform zoom
    var oKaMap = dd.elements.zoomTrack.div.kaZoomer.kaMap;
    var oMap = oKaMap.getCurrentMap();
    var nCurrentScale = oMap.getScales()[oMap.aScales.length - nNearestIndex - 1];
    oKaMap.zoomToScale(nCurrentScale);
}

//zoom to the level clicked in the track
function kaZoomer_zoomTo( e )
{
    e = (e)?e:((event)?event:null);
    var nClickTop = (e.layerY)?e.layerY:e.offsetY;
    //find current track height
    var oKaZoomer = dd.elements.zoomTrack.div.kaZoomer;
    var oKaMap = oKaZoomer.kaMap;
    var oMap = oKaMap.getCurrentMap();
    var nScales = oMap.getScales().length;
    var nTrackHeight = dd.Int(oKaZoomer.zoomTrackHeight) * nScales;

    //zoome to closest scale
    var nNearestIndex = Math.floor(nClickTop / nTrackHeight * nScales);
    var nNewScale = oMap.getScales()[oMap.aScales.length - nNearestIndex - 1];
    oKaMap.zoomToScale(nNewScale);
}

function kaZoomer_onmouseover( e )
{
    this.style.opacity = 1;
    this.style.mozOpacity = 1;
    this.style.filter = "Alpha(opacity=100)";
}

function kaZoomer_onmouseout( e )
{
    this.style.opacity = this.kaZoomer.opacity/100;
    this.style.mozOpacity = this.kaZoomer.opacity/100;
    this.style.filter = "Alpha(opacity="+this.kaZoomer.opacity+")";
}

function kaZoomer_zoomIn()
{
    this.kaZoomer.kaMap.zoomIn();
}

function kaZoomer_zoomOut()
{
    this.kaZoomer.kaMap.zoomOut();
}

function kaZoomer_alert()
{
    alert('here');
}
